#include "title.h"
#include "ingame.h"
#include "../world.h"
#include "../player.h"

#include "twn_game_api.h"

#include <SDL2/SDL.h>

#include <stdio.h>


static void title_tick(State *state) {
    SceneTitle *scn = (SceneTitle *)state->scene;
    (void)scn;

    input_action("ui_accept", CONTROL_RETURN);

    if (input_action_just_pressed("ui_accept")) {
        switch_to(state, ingame_scene);
        return;
    }

    m_sprite("/assets/title.png", ((Rect) {
            ((float)ctx.resolution.x / 2) - ((float)320 / 2), 64, 320, 128 }));
    
    /* draw the tick count as an example of dynamic text */
    size_t text_str_len = snprintf(NULL, 0, "%llu", (unsigned long long)state->ctx->frame_number) + 1;
    char *text_str = cmalloc(text_str_len);
    snprintf(text_str, text_str_len, "%llu", (unsigned long long)state->ctx->frame_number);

    const char *font = "fonts/kenney-pixel.ttf";
    float text_h = 32;
    float text_w = draw_text_width(text_str, text_h, font);

    draw_rectangle(
        (Rect) {
            .x = 0,
            .y = 0,
            .w = (float)text_w,
            .h = (float)text_h,
        },
        (Color) { 0, 0, 0, 255 }
    );
    draw_text(
        text_str,
        (Vec2){ 0, 0 },
        text_h,
        (Color) { 255, 255, 255, 255 },
        font
    );
    free(text_str);
}


static void title_end(State *state) {
    SceneTitle *scn = (SceneTitle *)state->scene;
    player_destroy(scn->player);
    world_destroy(scn->world);
    free(state->scene);
}


Scene *title_scene(State *state) {
    (void)state;

    SceneTitle *new_scene = ccalloc(1, sizeof *new_scene);
    new_scene->base.tick = title_tick;
    new_scene->base.end = title_end;

    new_scene->world = world_create();
    new_scene->player = player_create(new_scene->world);

    return (Scene *)new_scene;
}
